@@ -1,16 +1,3 @@
-/******************************************************************************
-*******************************************************************************
-**
-**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
-**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
-**
-**  This copyrighted material is made available to anyone wishing to use,
-**  modify, copy, or redistribute it subject to the terms and conditions
-**  of the GNU General Public License v.2.
-**
-*******************************************************************************
-******************************************************************************/
-
 #include <linux/sched.h>
 #include <linux/slab.h>
 #include <linux/smp_lock.h>
@@ -18,6 +5,7 @@
 #include <asm/semaphore.h>
 #include <linux/completion.h>
 #include <linux/buffer_head.h>
+#include <linux/exportfs.h>
 
 #include "gfs.h"
 #include "dio.h"
@@ -42,52 +30,6 @@ struct get_name_filldir
 	char *name;
 };
 
-/**
- * gfs_decode_fh -
- * @param1: description
- * @param2: description
- * @param3: description
- *
- * Function description
- *
- * Returns: what is returned
- */
-
-struct dentry *
-gfs_decode_fh(struct super_block *sb, __u32 *fh, int fh_len, int fh_type,
-	      int (*acceptable)(void *context, struct dentry *dentry),
-	      void *context)
-{
-	struct inode_cookie this, parent;
-
-	atomic_inc(&get_v2sdp(sb)->sd_ops_export);
-
-	if (fh_type != fh_len)
-		return NULL;
-
-	memset(&parent, 0, sizeof(struct inode_cookie));
-
-	switch (fh_type) {
-	case 6:
-		parent.gen_valid = TRUE;
-		parent.gen = gfs32_to_cpu(fh[5]);
-	case 5:
-		parent.formal_ino = ((uint64_t)gfs32_to_cpu(fh[3])) << 32;
-		parent.formal_ino |= (uint64_t)gfs32_to_cpu(fh[4]);
-	case 3:
-		this.gen_valid = TRUE;
-		this.gen = gfs32_to_cpu(fh[2]);
-		this.formal_ino = ((uint64_t)gfs32_to_cpu(fh[0])) << 32;
-		this.formal_ino |= (uint64_t)gfs32_to_cpu(fh[1]);
-		break;
-	default:
-		return NULL;
-	}
-
-	return gfs_export_ops.find_exported_dentry(sb, &this, &parent,
-						   acceptable, context);
-}
-
 /**
  * gfs_encode_fh -
  * @param1: description
@@ -257,12 +199,12 @@ gfs_get_parent(struct dentry *child)
 
 	ip = get_gl2ip(i_gh.gh_gl);
 
-	gfs_glock_dq_uninit(&d_gh);
-	gfs_glock_dq_uninit(&i_gh);
-
 	inode = gfs_iget(ip, CREATE);
 	gfs_inode_put(ip);
 
+	gfs_glock_dq_uninit(&d_gh);
+	gfs_glock_dq_uninit(&i_gh);
+
 	if (!inode)
 		return ERR_PTR(-ENOMEM);
 
@@ -271,6 +213,7 @@ gfs_get_parent(struct dentry *child)
 		iput(inode);
 		return ERR_PTR(-ENOMEM);
 	}
+
 	dentry->d_op = &gfs_dops;
 	return dentry;
 
@@ -291,10 +234,9 @@ gfs_get_parent(struct dentry *child)
  */
 
 struct dentry *
-gfs_get_dentry(struct super_block *sb, void *inump)
+gfs_get_dentry(struct super_block *sb, struct inode_cookie *cookie)
 {
 	struct gfs_sbd *sdp = get_v2sdp(sb);
-	struct inode_cookie *cookie = (struct inode_cookie *)inump;
 	struct gfs_inum inum;
 	struct gfs_holder i_gh, ri_gh, rgd_gh;
 	struct gfs_rgrpd *rgd;
@@ -310,8 +252,7 @@ gfs_get_dentry(struct super_block *sb, void *inump)
 	if (!cookie->formal_ino ||
 	    cookie->formal_ino == sdp->sd_jiinode->i_num.no_formal_ino ||
 	    cookie->formal_ino == sdp->sd_riinode->i_num.no_formal_ino ||
-	    cookie->formal_ino == sdp->sd_qinode->i_num.no_formal_ino ||
-	    cookie->formal_ino == sdp->sd_linode->i_num.no_formal_ino)
+	    cookie->formal_ino == sdp->sd_qinode->i_num.no_formal_ino)
 		return ERR_PTR(-EINVAL);
 
 	inum.no_formal_ino = cookie->formal_ino;
@@ -371,11 +312,11 @@ gfs_get_dentry(struct super_block *sb, void *inump)
 	atomic_inc(&sdp->sd_fh2dentry_misses);
 
  out:
-	gfs_glock_dq_uninit(&i_gh);
-
 	inode = gfs_iget(ip, CREATE);
 	gfs_inode_put(ip);
 
+	gfs_glock_dq_uninit(&i_gh);
+
 	if (!inode)
 		return ERR_PTR(-ENOMEM);
 
@@ -408,11 +349,55 @@ gfs_get_dentry(struct super_block *sb, void *inump)
 	return ERR_PTR(error);
 }
 
-struct export_operations gfs_export_ops = {
-	.decode_fh = gfs_decode_fh,
+static struct dentry *gfs_fh_to_dentry(struct super_block *sb, struct fid *fid,
+		int fh_len, int fh_type)
+{
+	struct inode_cookie this;
+	__u32 *fh = fid->raw;
+
+	atomic_inc(&get_v2sdp(sb)->sd_ops_export);
+
+	switch (fh_type) {
+	case 6:
+	case 5:
+	case 3:
+		this.gen_valid = TRUE;
+		this.gen = gfs32_to_cpu(fh[2]);
+		this.formal_ino = ((uint64_t)gfs32_to_cpu(fh[0])) << 32;
+		this.formal_ino |= (uint64_t)gfs32_to_cpu(fh[1]);
+		return gfs_get_dentry(sb, &this);
+	default:
+		return NULL;
+	}
+}
+
+static struct dentry *gfs_fh_to_parent(struct super_block *sb, struct fid *fid,
+		int fh_len, int fh_type)
+{
+	struct inode_cookie parent;
+	__u32 *fh = fid->raw;
+
+	atomic_inc(&get_v2sdp(sb)->sd_ops_export);
+
+	switch (fh_type) {
+	case 6:
+		parent.gen_valid = TRUE;
+		parent.gen = gfs32_to_cpu(fh[5]);
+	case 5:
+		parent.formal_ino = ((uint64_t)gfs32_to_cpu(fh[3])) << 32;
+		parent.formal_ino |= (uint64_t)gfs32_to_cpu(fh[4]);
+	default:
+		return NULL;
+	}
+
+	return gfs_get_dentry(sb, &parent);
+}
+
+const struct export_operations gfs_export_ops = {
 	.encode_fh = gfs_encode_fh,
+	.fh_to_dentry = gfs_fh_to_dentry,
+	.fh_to_parent = gfs_fh_to_parent,
 	.get_name = gfs_get_name,
 	.get_parent = gfs_get_parent,
-	.get_dentry = gfs_get_dentry,
 };
 
