@@ -1,28 +1,5 @@
 #!/bin/bash
 
-#
-#  Copyright Red Hat, Inc. 2006
-#
-#  This program is free software; you can redistribute it and/or modify it
-#  under the terms of the GNU General Public License as published by the
-#  Free Software Foundation; either version 2, or (at your option) any
-#  later version.
-#
-#  This program is distributed in the hope that it will be useful, but
-#  WITHOUT ANY WARRANTY; without even the implied warranty of
-#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-#  General Public License for more details.
-#
-#  You should have received a copy of the GNU General Public License
-#  along with this program; see the file COPYING.  If not, write to the
-#  Free Software Foundation, Inc.,  675 Mass Ave, Cambridge, 
-#  MA 02139, USA.
-#
-#
-#  Author(s):
-#	Marek Grac (mgrac at redhat.com)
-#
-
 export LC_ALL=C
 export LANG=C
 export PATH=/bin:/sbin:/usr/bin:/usr/sbin
@@ -102,10 +79,10 @@ generate_config_file()
 	done
 
 	generate_configTemplate "$generated_file" "$1"
-	echo "external_pid_file = '$PSQL_pid_file'" >> $generated_file
-	echo "listen_address = '$ip_comma'" >> $generated_file
+	echo "external_pid_file = '$PSQL_pid_file'" >> "$generated_file"
+	echo "listen_address = '$ip_comma'" >> "$generated_file"
 
-	echo >> $generated_file	
+	echo >> "$generated_file"	
 	sed 's/^[[:space:]]*external_pid_file/### external_pid_file/i;s/^[[:space:]]*listen_address/### listen_address/i' < "$original_file" >> "$generated_file"
 	
         sha1_addToFile "$generated_file"
